<?php

namespace Bixo\Account\Http\Controllers;

use Exception;
use Bixo\Account\Actions\AccountWorkflow;
use Bixo\Account\Http\Requests\AccountWorkflowRequest;
use Bixo\Account\Http\Resources\AccountResource;
use Bixo\Account\Models\Account;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class AccountWorkflowController extends BaseController
{
    /**
     * Action controller function for account.
     *
     * @param Model $account
     * @param action next action for the account.
     *
     * @return Response
     */
    public function __invoke(AccountWorkflowRequest $request, Account $account, $trasition)
    {
        try {
            $request = $request->all();
            $account = AccountWorkflow::run($trasition, $account, $request);
            $data = new AccountResource($account);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::account.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/account/' . $account->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/account/' . $account->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the account and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $account
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $account, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $account = Account::findBySignedId($account);
        $transitions = $this->transitions($account, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('account::account.name') . ' ' . trans('Approvals'))
            ->view('account::public.account.workflow')
            ->layout('mobile')
            ->data(compact('account', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the account.
     *
     * @param PublicRequest $request
     * @param string $account
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $account, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $account = Account::findBySignedId($account);
        $user = user();
        $account = AccountWorkflow::run($transition, $account, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($account, $user)
    {
        $transitions = [];
        foreach ($account->workflow()->transitions($account) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.account.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'account' => $account->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $account->workflow()->form($value);
            $array['label'] = trans('account::account.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
