<?php

namespace Bixo\Account\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Account\Forms\Allocation as AllocationForm;
use Bixo\Account\Http\Requests\AllocationResourceRequest;
use Bixo\Account\Http\Resources\AllocationResource;
use Bixo\Account\Http\Resources\AllocationsCollection;
use Bixo\Account\Models\Allocation;
use Bixo\Account\Actions\AllocationAction;
use Bixo\Account\Actions\AllocationActions;


/**
 * Resource controller class for allocation.
 */
class AllocationResourceController extends BaseController
{

    /**
     * Initialize allocation resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = AllocationForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.account.modules'), 'account', guard_url('account'));
            return $next($request);
        });
    }

    /**
     * Display a list of allocation.
     *
     * @return Response
     */
    public function index(AllocationResourceRequest $request)
    {
        $request = $request->all();
        $page = AllocationActions::run('paginate', $request);

        $data = new AllocationsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('account::allocation.names'))
            ->view('account::allocation.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display allocation.
     *
     * @param Request $request
     * @param Model   $allocation
     *
     * @return Response
     */
    public function show(AllocationResourceRequest $request, Allocation $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AllocationResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('account::allocation.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('account::allocation.show')
            ->output();
    }

    /**
     * Show the form for creating a new allocation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AllocationResourceRequest $request, Allocation $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AllocationResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::allocation.name'))
            ->view('account::allocation.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new allocation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AllocationResourceRequest $request, Allocation $model)
    {
        try {
            $request = $request->all();
            $model = AllocationAction::run('store', $model, $request);
            $data = new AllocationResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::allocation.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/allocation/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/allocation'))
                ->redirect();
        }

    }

    /**
     * Show allocation for editing.
     *
     * @param Request $request
     * @param Model   $allocation
     *
     * @return Response
     */
    public function edit(AllocationResourceRequest $request, Allocation $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AllocationResource($model);
        // return view('account::allocation.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::allocation.name'))
            ->view('account::allocation.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the allocation.
     *
     * @param Request $request
     * @param Model   $allocation
     *
     * @return Response
     */
    public function update(AllocationResourceRequest $request, Allocation $model)
    {
        try {
            $request = $request->all();
            $model = AllocationAction::run('update', $model, $request);
            $data = new AllocationResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::allocation.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/allocation/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/allocation/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the allocation.
     *
     * @param Model   $allocation
     *
     * @return Response
     */
    public function destroy(AllocationResourceRequest $request, Allocation $model)
    {
        try {

            $request = $request->all();
            $model = AllocationAction::run('destroy', $model, $request);
            $data = new AllocationResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::allocation.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/allocation/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/allocation/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
