<?php

namespace Bixo\Account\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Account\Forms\Bank as BankForm;
use Bixo\Account\Http\Requests\BankResourceRequest;
use Bixo\Account\Http\Resources\BankResource;
use Bixo\Account\Http\Resources\BanksCollection;
use Bixo\Account\Models\Bank;
use Bixo\Account\Actions\BankAction;
use Bixo\Account\Actions\BankActions;


/**
 * Resource controller class for bank.
 */
class BankResourceController extends BaseController
{

    /**
     * Initialize bank resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = BankForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.account.modules'), 'account', guard_url('account'));
            return $next($request);
        });
    }

    /**
     * Display a list of bank.
     *
     * @return Response
     */
    public function index(BankResourceRequest $request)
    {
        $request = $request->all();
        $page = BankActions::run('paginate', $request);

        $data = new BanksCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('account::bank.names'))
            ->view('account::bank.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display bank.
     *
     * @param Request $request
     * @param Model   $bank
     *
     * @return Response
     */
    public function show(BankResourceRequest $request, Bank $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BankResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('account::bank.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('account::bank.show')
            ->output();
    }

    /**
     * Show the form for creating a new bank.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BankResourceRequest $request, Bank $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BankResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::bank.name'))
            ->view('account::bank.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new bank.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BankResourceRequest $request, Bank $model)
    {
        try {
            $request = $request->all();
            $model = BankAction::run('store', $model, $request);
            $data = new BankResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::bank.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/bank/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/bank'))
                ->redirect();
        }

    }

    /**
     * Show bank for editing.
     *
     * @param Request $request
     * @param Model   $bank
     *
     * @return Response
     */
    public function edit(BankResourceRequest $request, Bank $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BankResource($model);
        // return view('account::bank.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::bank.name'))
            ->view('account::bank.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the bank.
     *
     * @param Request $request
     * @param Model   $bank
     *
     * @return Response
     */
    public function update(BankResourceRequest $request, Bank $model)
    {
        try {
            $request = $request->all();
            $model = BankAction::run('update', $model, $request);
            $data = new BankResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::bank.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/bank/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/bank/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the bank.
     *
     * @param Model   $bank
     *
     * @return Response
     */
    public function destroy(BankResourceRequest $request, Bank $model)
    {
        try {

            $request = $request->all();
            $model = BankAction::run('destroy', $model, $request);
            $data = new BankResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::bank.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('account/bank/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/bank/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
