<?php

namespace Bixo\Account\Http\Controllers;

use Exception;
use Bixo\Account\Actions\BankWorkflow;
use Bixo\Account\Http\Requests\BankWorkflowRequest;
use Bixo\Account\Http\Resources\BankResource;
use Bixo\Account\Models\Bank;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class BankWorkflowController extends BaseController
{
    /**
     * Action controller function for bank.
     *
     * @param Model $bank
     * @param action next action for the bank.
     *
     * @return Response
     */
    public function __invoke(BankWorkflowRequest $request, Bank $bank, $trasition)
    {
        try {
            $request = $request->all();
            $bank = BankWorkflow::run($trasition, $bank, $request);
            $data = new BankResource($bank);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::bank.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/bank/' . $bank->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/bank/' . $bank->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the bank and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $bank
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $bank, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $bank = Bank::findBySignedId($bank);
        $transitions = $this->transitions($bank, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('account::bank.name') . ' ' . trans('Approvals'))
            ->view('account::public.bank.workflow')
            ->layout('mobile')
            ->data(compact('bank', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the bank.
     *
     * @param PublicRequest $request
     * @param string $bank
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $bank, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $bank = Bank::findBySignedId($bank);
        $user = user();
        $bank = BankWorkflow::run($transition, $bank, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($bank, $user)
    {
        $transitions = [];
        foreach ($bank->workflow()->transitions($bank) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.bank.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'bank' => $bank->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $bank->workflow()->form($value);
            $array['label'] = trans('account::bank.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
