<?php

namespace Bixo\Account\Http\Controllers;

use Exception;
use Bixo\Account\Actions\CustomerWorkflow;
use Bixo\Account\Http\Requests\CustomerWorkflowRequest;
use Bixo\Account\Http\Resources\CustomerResource;
use Bixo\Account\Models\Customer;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class CustomerWorkflowController extends BaseController
{
    /**
     * Action controller function for customer.
     *
     * @param Model $customer
     * @param action next action for the customer.
     *
     * @return Response
     */
    public function __invoke(CustomerWorkflowRequest $request, Customer $customer, $trasition)
    {
        try {
            $request = $request->all();
            $customer = CustomerWorkflow::run($trasition, $customer, $request);
            $data = new CustomerResource($customer);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::customer.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/customer/' . $customer->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/customer/' . $customer->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the customer and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $customer
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $customer, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $customer = Customer::findBySignedId($customer);
        $transitions = $this->transitions($customer, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('account::customer.name') . ' ' . trans('Approvals'))
            ->view('account::public.customer.workflow')
            ->layout('mobile')
            ->data(compact('customer', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the customer.
     *
     * @param PublicRequest $request
     * @param string $customer
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $customer, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $customer = Customer::findBySignedId($customer);
        $user = user();
        $customer = CustomerWorkflow::run($transition, $customer, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($customer, $user)
    {
        $transitions = [];
        foreach ($customer->workflow()->transitions($customer) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.customer.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'customer' => $customer->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $customer->workflow()->form($value);
            $array['label'] = trans('account::customer.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
