<?php

namespace Bixo\Account\Http\Controllers;

use Exception;
use Bixo\Account\Actions\PayrecWorkflow;
use Bixo\Account\Http\Requests\PayrecWorkflowRequest;
use Bixo\Account\Http\Resources\PayrecResource;
use Bixo\Account\Models\Payrec;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class PayrecWorkflowController extends BaseController
{
    /**
     * Action controller function for payrec.
     *
     * @param Model $payrec
     * @param action next action for the payrec.
     *
     * @return Response
     */
    public function __invoke(PayrecWorkflowRequest $request, Payrec $payrec, $trasition)
    {
        try {
            $request = $request->all();
            $payrec = PayrecWorkflow::run($trasition, $payrec, $request);
            $data = new PayrecResource($payrec);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::payrec.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('account/payrec/' . $payrec->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/payrec/' . $payrec->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the payrec and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $payrec
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $payrec, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $payrec = Payrec::findBySignedId($payrec);
        $transitions = $this->transitions($payrec, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('account::payrec.name') . ' ' . trans('Approvals'))
            ->view('account::public.payrec.workflow')
            ->layout('mobile')
            ->data(compact('payrec', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the payrec.
     *
     * @param PublicRequest $request
     * @param string $payrec
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $payrec, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $payrec = Payrec::findBySignedId($payrec);
        $user = user();
        $payrec = PayrecWorkflow::run($transition, $payrec, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($payrec, $user)
    {
        $transitions = [];
        foreach ($payrec->workflow()->transitions($payrec) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.payrec.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'payrec' => $payrec->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $payrec->workflow()->form($value);
            $array['label'] = trans('account::payrec.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
